<?php
/**
 * Class to call the router routino from php
 *
 * PHP version 5.1.0+
 *
 * Copyright (c) 2012, Uwe Steinmann
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  - Neither the name of the The PEAR Group nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @package   Router_Routino
 * @author    Uwe Steinmann <steinm@php.net>
 * @copyright 2012 Uwe Steinmann
 * @license   http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @version   CVS: $Revision: $
 * @link      http://pear.php.net/package/Router_Routino
 */

/**
 * Include PEAR package for error handling
 */
require_once 'PEAR.php';

/**
 * Include Router_Routino_Result class
 */
require_once 'Router_Routino/Router_Routino/Result.php';


/**
 * Class for setting the parameters and running routino
 *
 * @package   Router_Routino
 * @author    Uwe Steinmann <steinm@php.net>
 * @copyright 2012 Uwe Steinmann
 * @license   http://www.opensource.org/licenses/bsd-license.php New BSD License
 * @link      http://pear.php.net/package/Router_Routino
 */
class Router_Routino {
  /**
   * @var string $datadir directory containing the database
   * @access protected
   */
	protected $datadir;

  /**
   * @var string $prefix prefix of the database files
   * @access protected
   */
	protected $prefix;

  /**
   * @var string $language 2 letter language code
   * @access protected
   */
	protected $language;

  /**
   * @var array $postions list of positions
   * @access protected
   */
	protected $postions;

  /**
   * @var array $allowedhighways list valid highway types
   * @access public
   */
	public static $allowedhighways = array(
  	'motorway',
  	'trunk',
  	'primary',
  	'secondary',
  	'tertiary',
  	'unclassified',
  	'residential',
  	'service',
  	'track',
  	'cycleway',
  	'path',
  	'steps',
  	'ferry',
	);

  /**
   * Constructor
   * 
   * @param string $datadir directory containing the database
   * @param string $prefix prefix of the database files
   */
	function __construct($datadir='/var/lib/routino/data', $prefix='') { /* {{{ */
		$this->datadir = $datadir;
		$this->prefix = $prefix;
		$this->positions = array();
		$this->language = '';
		$this->highways = array();
		$this->speed = array();
	} /* }}} */

  /**
   * Set language of result
   * 
   * @param string $lang two letter language code
	 */
	function setLanguage($lang) { /* {{{ */
		$this->language = substr($lang, 0, 2);
	} /* }}} */

  /**
   * Set preference value for highways
   * 
   * @param string $type type of highway
   * @param int $percentage preference value from 0-100
	 */
	function setHighwayPreference($type, $percentage) { /* {{{ */
		if(!in_array($type, self::$allowedhighways))
			return false;
		if($percentage > 100)
			$percentage = 100;
		if($percentage < 0)
			$percentage = 0;
		$this->highway[$type] = (int) $percentage;
	} /* }}} */

  /**
   * Set maximum speed for highways
   * 
   * @param string $type type of highway
   * @param int $maxspeed maximum speed
	 */
	function setHighwayMaxspeed($type, $maxspeed) { /* {{{ */
		if(!in_array($type, self::$allowedhighways))
			return false;
		if($maxspeed < 0)
			$maxspeed = 0;
		$this->speed[$type] = (int) $maxspeed;
	} /* }}} */

	/**
	 * Adds another position after the last one
	 *
	 * @param float $lat latitude of new position
	 * @param float $lon longitude of new position
	 */
	function addPosition($lat, $lon) { /* {{{ */
		$this->positions[] = array('lat'=>$lat, 'lon'=>$lon);
	} /* }}} */

	/**
	 * Remove all positions previously set with addPosition()
	 *
	 */
	function clearPositions() { /* {{{ */
		$this->positions = array();
	} /* }}} */

  /**
   * Constructor
   * 
   * @param string $transportation type of transportation
   * @param string $type shortest or quiekest route
   * @return object result object
   */
	function run($transportation='motorcar', $type='shortest') { /* {{{ */
		if(count($this->positions) < 2) {
			return false;
		}
		if($type != 'shortest' && $type != 'quickest') {
			return false;
		}

		$tmpdir = sys_get_temp_dir().'/'.uniqid();
		if(!mkdir($tmpdir))
			return false;
		$cmd = 'cd '.$tmpdir.';';
		$cmd .= ' routino-router --dir='.$this->datadir.' --transport='.$transportation.' --'.$type;
		if($this->language)
			$cmd = ' --language='.$this->language;
		$i = 1;
		foreach($this->positions as $pos) {
			$cmd .= ' --lon'.$i.'='.number_format($pos['lon'], 6, '.', '').' --lat'.$i.'='.number_format($pos['lat'], 6, '.', '');
			$i++;
		}
 		$cmd .= ' --quiet';
		if($this->highways) {
			foreach($this->highways as $type2=>$percentage)
				$cmd .= ' --highway-'.$type2.'='.$percentage;
		}
		if($this->speed) {
			foreach($this->speed as $type2=>$maxspeed)
				$cmd .= ' --speed-'.$type2.'='.$maxspeed;
		}

		exec($cmd);
		$result = new Router_Routino_Result($tmpdir, $type);
		exec('rm -rf '.$tmpdir);

		return $result;
	} /* }}} */
}


?>
